#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

__author__ = "Mikey Garcia, @gikeymarcia"
from pydymenu.fzf import FzfProtocol
from pydymenu.fzf import fzf_func as fzf
from pydymenu.rofi import RofiProtocol
from pydymenu.rofi import rofi_func as rofi

__author__ = "Mikey Garcia, @gikeymarcia"
__version__ = "0.5.1"

__all__ = ["fzf", "FzfProtocol", "rofi", "RofiProtocol"]



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

from typing import Protocol, List, Iterable, Optional


class Menu(Protocol):
    def __init__(
        self,
        items: Iterable[str],
        prompt: str,
        multi: bool,
        case_sensitive: bool,
        **kwargs
    ):
        raise NotImplementedError

    def select(self) -> Optional[List[str]]:
        raise NotImplementedError



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

from random import shuffle
import time
from typing import Iterable

__all__ = ["list_options", "gen_options"]

sensemakers = [
    "Jordan Hall",
    "Bret Weinstein",
    "Jim Rutt",
    "John Vervaeke",
    "Jamie Wheal",
    "The Stoa",
    "Jonathan Pageau",
    "Jordan Peterson",
    "Joe Rogan",
    "Sam Harris",
    "Terrence McKenna",
    "Rupert Spira",
    "Curt Jaimungal",
    "Daryl Davis",
    "Aaron Mate",
    "Abby Martin",
    "Lex Fridman",
    "Glenn Loury",
    "David Fuller",
    "Ken Wilber",
    "Yoga with Adrienne",
    "Jimmy Dore",
    "Dave Snowden",
    "Tristan Harris",
    "Paul Stamets",
    "Daniel Schmachtenberger",
    "Peter Wang",
    "John Robb",
    "Samo Burja",
    "Tyson Yunkaporta",
    "Michael Malice",
    "Joscha Bach",
    "Michel Bauwens",
    "Yancey Strickler",
    "Hanzi Freinacht",
]


def list_to_generator(source: Iterable[str]) -> Iterable[str]:
    for i in source:
        time.sleep(0.05)
        yield i


shuffle(sensemakers)
list_options = sensemakers
gen_options = list_to_generator(sensemakers)


# vim: foldlevel=4 :



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

from typing import Iterable, List, Optional

from pydymenu.exceptions import MissingProgram
from pydymenu.menu import Menu
from pydymenu.system import missing_binary, stream_to_stdin


class FzfProtocol(Menu):
    """Implements Menu protcol for `fzf` selector.

    Takes Iterable[str] as input and returns:
        List[str]:  upon selection
        None:       when no selection is made
    """

    def __init__(
        self,
        items: Iterable[str],
        prompt: str = None,
        multi: bool = False,
        case_sensitive: bool = False,
        preview: str = None,
    ):
        self.items = items
        self.prompt = " > " if prompt is None else prompt
        self.multi = multi
        self.case_sensitive = case_sensitive
        self.preview = preview
        self.flags: List[str] = self.process_opts()
        self.command: List[str] = ["fzf"] + self.flags
        if missing_binary("fzf"):
            raise MissingProgram("Cannot find `fzf` on your system.")

    def process_opts(self) -> List[str]:
        """Turn selected options into List[str] of `fzf` flags."""
        prompt = ["--prompt", self.prompt]
        multi = "--multi" if self.multi else "--no-multi"
        case = "+i" if self.case_sensitive else "-i"
        if self.preview:
            return prompt + [multi, case, "--preview", self.preview]
        else:
            return prompt + [multi, case]

    def select(self) -> Optional[List[str]]:
        """Run `fzf` selector on given items."""
        output: Optional[str] = stream_to_stdin(self.items, self.command)
        if output is None:
            return None
        else:
            return [line for line in output.split("\n") if len(line) > 0]


def fzf_func(
    items: Iterable[str],
    prompt: str = None,
    multi: bool = False,
    case_sensitive: bool = False,
    preview: str = None,
) -> Optional[List[str]]:
    """Launches a `fzf` process and returns either List[str] or None."""
    fuzzy_finder = FzfProtocol(
        items=items,
        prompt=prompt,
        multi=multi,
        case_sensitive=case_sensitive,
        preview=preview,
    )
    return fuzzy_finder.select()


# vim: foldlevel=1:



from rich.console import Console
from rich.theme import Theme

my_theme = Theme(
    {
        "good": "bold green",
        "bad": "underline red",
        "title": "bold blue",
    }
)

console = Console(theme=my_theme, color_system="auto")



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

import errno
from shutil import which
import subprocess as sp
from typing import Iterable, List, Optional


def missing_binary(binary_name: str) -> bool:
    """Tests whether a binary is missing from the system"""
    if isinstance(binary_name, str):
        return False if which(binary_name) else True
    else:
        raise ValueError("This function only accepts string inputs.")


def stream_to_stdin(items: Iterable[str], command: List[str]) -> Optional[str]:
    """
    Stream each Iteralbe item into the standard in for the given command.

    Each item is appended with a '\n' and utf-8' encoded before streamed in.

    Uses low-level subprocess.Popen(). If the process is interrupted or
    completes with an error code the None value will be returned. Otherwise,
    the byte output of Popen.stdout is decoded into a string and returned.

    """
    proc = sp.Popen(command, stdin=sp.PIPE, stdout=sp.PIPE, stderr=None)
    stdin, stdout = proc.stdin, proc.stdout
    line_return, new_line = "\r", "\n"
    if proc is not None and stdin is not None and stdout is not None:
        for i in items:
            if new_line in i or line_return in i:
                raise ValueError("no line breaks within an item.")
            else:
                try:
                    stdin.write(i.encode("utf-8") + b"\n")
                    stdin.flush()
                except IOError as e:
                    if e.errno != errno.EPIPE and errno.EPIPE != 32:
                        raise
            poll = proc.poll()
            if poll in [1, 2, 130]:
                # fzf returns 130 when interrupted with Ctrl+c
                return None
            elif poll == 0:
                return stdout.read().decode()
        try:
            stdin.close()
        except IOError as e:
            if e.errno != errno.EPIPE and errno.EPIPE != 32:
                raise
        if proc is None or proc.wait() not in [0, 1]:
            return None
        elif stdout is not None:
            return stdout.read().decode()
        else:
            return None


def newline_joined_bytestream(menu_items: Iterable[str]) -> bytes:
    """Takes an Iterable[str] and returns a bytestream suitable for standard in

    Joins all strings with '\n' then encodes as a utf-8 bytestream.
    Good for subprocess.run(input=) use
    """
    line_return, new_line = "\r", "\n"

    def ready_to_join(list_item: str) -> str:
        if new_line in list_item or line_return in list_item:
            err = f"Newlines not allowed within items. Found in\n{list_item}"
            raise ValueError(err)
        else:
            return list_item

    return "\n".join([ready_to_join(i) for i in menu_items]).encode("utf-8")


def command_input(_: List[str]) -> List[str]:
    """Run a shell command and get a result suitable for pydymenu input.

    For example:
    pydymenu.fzf(
        command_input(["find", Path.home(), "-name", "*.mp3"]),
        prompt="Which mp3(s) do you want?",
        multi=True
    )
    """
    # TODO make this
    raise NotImplementedError


__all__ = ["newline_joined_bytestream", "missing_binary", "stream_to_stdin"]



# https://realpython.com/pypi-publish-python-package/#a-small-python-package
import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()


# https://setuptools.readthedocs.io/en/latest/setuptools.html#basic-use
setup(
    name="pydymenu",
    version="0.5.1",
    description="A pythonic wrapper interface for fzf, dmenu, and rofi.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/gikeymarcia/pydymenu",
    author="Mikey Garcia",
    author_email="gikeymarcia@gmail.com",
    license="GPL-3.0",
    packages=find_packages(exclude="test"),
    install_requires=["rich"],
)



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

import pytest

import pydymenu
from pydymenu.demo_data import gen_options
from pydymenu.exceptions import MissingProgram
from pydymenu.system import missing_binary


def test_make_rofi_object():
    assert pydymenu.rofi


def test_rofi_present():
    assert missing_binary("rofi") == False


def test_rofi_absent(monkeypatch):
    monkeypatch.setenv("PATH", "")
    with pytest.raises(MissingProgram) as except_info:
        pydymenu.rofi(["one", "two", "three"])
    assert except_info.type is MissingProgram


@pytest.mark.parametrize(
    "kwargs, flags",
    [
        ({}, ["-p", " > ", "-i"]),
        ({"prompt": None}, ["-p", " > ", "-i"]),
        ({"prompt": "go larry!"}, ["-p", "go larry!", "-i"]),
        ({"case_sensitive": True}, ["-p", " > ", "-case-sensitive"]),
        ({"multi": True}, ["-p", " > ", "-multi-select", "-i"]),
        ({"multi": True, "prompt": "do it"}, ["-p", "do it", "-multi-select", "-i"]),
        (
            {"multi": True, "case_sensitive": True},
            ["-p", " > ", "-multi-select", "-case-sensitive"],
        ),
        # TODO ignore extra args? Not sure if this matters
        # ({"preview": "git diff HEAD -- {}"}, ["-p", " > "]),
    ],
)
def test_flag_calculation(kwargs, flags):
    rofi = pydymenu.RofiProtocol(gen_options, **kwargs)
    assert (rofi.flags) == flags



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

import argparse
from pathlib import Path

from pydymenu import fzf, rofi
from pydymenu.console import console
from pydymenu.demo_data import gen_options
from rich.syntax import Syntax

parser = argparse.ArgumentParser(
    prog="python -m pydymenu",
    description="Try pydymenu in either --fzf or --rofi mode.",
)
group = parser.add_mutually_exclusive_group()
group.add_argument("--fzf", help="Select using fzf", action="store_true")
group.add_argument("--rofi", help="Select using rofi", action="store_true")

args = parser.parse_args()
# console.log(args)

if args.fzf:
    fzf_selection = fzf(
        gen_options,
        prompt="Choose with fzf: ",
        multi=True,
        preview="echo {} | sed -E 's/[aeiou]//g' | figlet",
    )
    # show code sample
    console.print("FZF mode (sample code)\n", style="title")
    demo_file = Path(__file__).parent / "demo" / "fzf.py"
    with open(demo_file, "r") as file:
        # use pygments theme: https://pygments.org/demo/#try
        example = Syntax(demo_file.read_text(), "python", theme="material")
        console.print(example)
    console.print(f"{fzf_selection = }")
elif args.rofi:
    rofi_selection = rofi(
        gen_options,
        prompt="Choose with rofi: ",
        multi=True,
    )
    # show code sample
    console.print("rofi mode (sample code)\n", style="title")
    demo_file = Path(__file__).parent / "demo" / "rofi.py"
    with open(demo_file, "r") as file:
        example = Syntax(demo_file.read_text(), "python", theme="material")
        console.print(example)
    console.print(f"{rofi_selection = }")
else:
    args = parser.parse_args(["--help"])



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

from pydymenu.system import missing_binary
import pytest
from pathlib import Path


def test_has_bin():
    assert missing_binary("fzf") == False
    assert missing_binary("rofi") == False
    assert missing_binary("") == True
    assert missing_binary("printf") == False


@pytest.mark.parametrize("wrong_types", [None, {}, 127, Path.home()])
def test_missing_binary_throws_value_errors(wrong_types):
    with pytest.raises(ValueError) as except_info:
        missing_binary(wrong_types)
    assert except_info.type == ValueError


# vim: set foldlevel=2:



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu

import pytest

import pydymenu
from pydymenu.demo_data import gen_options
from pydymenu.exceptions import MissingProgram


def test_missing_fzf_error(monkeypatch):
    monkeypatch.setenv("PATH", "")
    with pytest.raises(MissingProgram) as except_info:
        pydymenu.fzf(["one"])
    assert except_info.type is MissingProgram


@pytest.mark.parametrize(
    "kwargs, flags",
    [
        ({}, ["--prompt", " > ", "--no-multi", "-i"]),
        ({"prompt": None}, ["--prompt", " > ", "--no-multi", "-i"]),
        ({"prompt": "go larry!"}, ["--prompt", "go larry!", "--no-multi", "-i"]),
        ({"case_sensitive": True}, ["--prompt", " > ", "--no-multi", "+i"]),
        ({"multi": True}, ["--prompt", " > ", "--multi", "-i"]),
        ({"multi": True, "case_sensitive": True}, ["--prompt", " > ", "--multi", "+i"]),
        (
            {"preview": "git diff HEAD -- {}"},
            ["--prompt", " > ", "--no-multi", "-i", "--preview", "git diff HEAD -- {}"],
        ),
        (
            {"preview": "git diff HEAD -- {}", "multi": True},
            ["--prompt", " > ", "--multi", "-i", "--preview", "git diff HEAD -- {}"],
        ),
    ],
)
def test_flag_calculation(kwargs, flags):
    fzf = pydymenu.FzfProtocol(gen_options, **kwargs)
    assert (fzf.flags) == flags


# vim: set foldlevel=2:



#!/usr/bin/env python3
# Mikey Garcia, @gikeymarcia
# https://github.com/gikeymarcia/pydymenu


class MissingProgram(Exception):
    """Exception raised when a required binary is missing from the system."""

    pass
